import cwbplot
from pathlib import Path
import matplotlib
import os
from matplotlib.font_manager import FontProperties


__version__= "0.0.7"

verlist = matplotlib.__version__.split(".")
_cntjude4matplt = 0
if int(verlist[0]) < 3:
    _cntjude4matplt = 1
if _cntjude4matplt == 0 and int(verlist[1]) < 2:
    _cntjude4matplt = 1

cwbpakpath = cwbplot.__path__[0]
#bentham blue-highway caliban carlito expressway-free raleway rosario TaipeiSansTCBeta times
def font(lang = "en", style = "times", weight="normal", size = 12):
    if lang.lower() == "en":
        if _cntjude4matplt == 0 and weight == "normal":
            getfont = Path(os.path.join(cwbpakpath, f"fonts/{style}.ttf"))
        elif _cntjude4matplt == 0 and weight == "bold":
            getfont = Path(os.path.join(cwbpakpath, f"fonts/{style}bd.ttf"))
        elif _cntjude4matplt == 1 and weight == "normal":
            getfont = os.path.join(cwbpakpath, f"fonts/{style}.ttf")
        elif _cntjude4matplt == 1 and weight == "bold":
            getfont = os.path.join(cwbpakpath, f"fonts/{style}bd.ttf")
    if lang.lower() == "cht":
        if _cntjude4matplt == 0 and weight == "normal":
            getfont = Path(os.path.join(cwbpakpath, "fonts/TaipeiSansTCBeta.ttf"))
        elif _cntjude4matplt == 0 and weight == "bold":
            getfont = Path(os.path.join(cwbpakpath, "fonts/TaipeiSansTCBetabd.ttf"))
        elif _cntjude4matplt == 1 and weight == "normal":
            getfont = os.path.join(cwbpakpath, "fonts/TaipeiSansTCBeta.ttf")
        elif _cntjude4matplt == 1 and weight == "bold":
            getfont = os.path.join(cwbpakpath, "fonts/TaipeiSansTCBetabd.ttf")
    if _cntjude4matplt == 0:
        fontdict = {"fname":getfont,"size":size}
    else:
        fontdict = FontProperties(fname=getfont,size=size)
    return fontdict



import netCDF4 as nc
import matplotlib.pyplot as plt                                                     
import numpy as np
from matplotlib.cm import get_cmap
from matplotlib.widgets import Button
from mpl_toolkits.basemap import Basemap
from wrf import  getvar, get_basemap, to_np, latlon_coords, CoordPair, vertcross
def get_layer_from_varname(var_name):
    if ("#" in var_name): 
      var_name, layer = var_name.split("#",-1)
      layer = int(layer)
      if (layer <0): 
        print("layer index less than zero")
    else:
      layer = 0
    return var_name, layer 
def  diff_index(diff,i):
     """ return boolen for difference or not """
     if(diff==-1):
        return True
     elif (diff==0):	
        return False
     else:	
        for elements in diff:
            if (elements==i+1): return True
        return False	    
    
def nc_diff(nc_f1, nc_f2,var_list, nrows=0,var_cmap=None, bm=None, diff=-1, mtitle=None, density=10):
  """
  Quick difference for two nc files for example
  nc_diff(nc_fid, nc_fid1,var_list, nrows=0,var_cmap=None)
  Aruguments:
  nc_fid1:first nc-file id,object from nc_fid=nc.Dataset("filename",'r') 
  nc_fid2:second nc-file id, same type as nc_fid
  var_list: string sequence of variable. var_list="T2 U10 V10"
  nrows: number of rows for subplot, defualt is automatic adjustment(nrows=0).
  var_cmap: string sequence of varcmap, which is same size as var_list, default is "jet". var_cmap="jet coolwarm Spectral"
  bm: Object from Basemap module. define plot area settings
  diff: index of subplots should be differnce for exmaple 3X2 subplot.ex: diff=1,2,5 
                                  |_1_|_2_|_3_|
				  |_4_|_5_|_6_|
        -1:all subplots are difference
	0 :all subplots are first file variables
  mtitle: index of main title for two files directory default("None")
          for example mtitle=(-4, -3) 
  nc_f1 = '/data2/datafdda/xa36/EX_SFCDA/CNTL/ncdf/17070603/wrfout_d01_2017-07-06_03:00:00'
                                          -4   -3    -2              -1
  nc_f2 = '/data2/datafdda/xa36/EX_SFCDA/3DVAR/SFC/17070603/wrfout_d01_2017-07-06_03:00:00'
                                               -3    -2         -1
            may print out main title " CNTL v.s. SFC "					        
  density: vectory density for wind vector plot default is 10
	

  """

  var_name = var_list.split()
  nvar = len(var_name)
  var = list()

  if (var_cmap is None):
     print("no_var_cmap")
     var_cmap = 'jet '*nvar

  var_cmap = var_cmap.split()

  ## Read NC FILE
  nc_fid1=nc.Dataset(nc_f1,'r')
  nc_fid2=nc.Dataset(nc_f2,'r')

  for i in range(nvar):
     print(var_name[i])
     var_name[i], level = get_layer_from_varname(var_name[i])
     if("." in var_name[i]):
        nfile = int(var_name[i].split(".",-1)[-1])
        if(nfile==2):
           nc_read = nc_fid2
        else:
           nc_read = nc_fid1
        var_name[i] = var_name[i].split(".",-1)[0]
     else:
       nc_read=nc_fid1
       nfile = 1
     if ("wspd" in var_name[i]):
        if ("Z" in getvar(nc_read,var_name[i]).MemoryOrder):
          print("please specify the level")
#          var.append(getvar(nc_read,var_name[i])[0][level,:,:])
          var=getvar(nc_read,var_name[i], units="m s-1")[0][level,:,:]
          data = getvar(nc_read,var_name[i], units="m s-1")[0][level,:,:].data

          if diff_index(diff,i):
             data1 = getvar(nc_fid2,var_name[i], units="m s-1")[0][level,:,:].data
  
        else:
#          var.append(getvar(nc_read,var_name[i], units="m s-1")[0])
          var=(getvar(nc_read,var_name[i], units="m s-1")[0])
          data = getvar(nc_read,var_name[i], units="m s-1")[0].data

          if diff_index(diff,i):
             data1 = getvar(nc_fid2,var_name[i], units="m s-1")[0].data
    
     elif ("uvmet" in var_name[i]):
        if ("Z" in getvar(nc_read,var_name[i]).MemoryOrder):
          print("please specify the level")
#          var.append(getvar(nc_read,var_name[i])[:,level,:,:])
          var=(getvar(nc_read,var_name[i])[:,level,:,:])
          data = getvar(nc_read,var_name[i])[:,level,:,:].data

          if diff_index(diff,i):
             data1 = getvar(nc_fid2,var_name[i])[:,level,:,:].data
  
        else:
#          var.append(getvar(nc_read,var_name[i], units="m s-1"))
          var=(getvar(nc_read,var_name[i], units="m s-1"))
          data = getvar(nc_read,var_name[i], units="m s-1").data

          if diff_index(diff,i):
             data1 = getvar(nc_fid2,var_name[i], units="m s-1").data
    
     else:
       if ("Z" in getvar(nc_read,var_name[i]).MemoryOrder):
          print("please specify the level")
#          var.append(getvar(nc_read,var_name[i])[level,:,:])
          var=(getvar(nc_read,var_name[i])[level,:,:])
          data = getvar(nc_read,var_name[i])[level,:,:].data

          if diff_index(diff,i):
             data1 = getvar(nc_fid2,var_name[i])[level,:,:].data
       else:	
#          var.append(getvar(nc_read,var_name[i]))
          var=(getvar(nc_read,var_name[i]))
          data = getvar(nc_read,var_name[i]).data

          if diff_index(diff,i):
             data1 = getvar(nc_fid2,var_name[i]).data

     if diff_index(diff,i):
        val_diff = data-data1
        var.data=val_diff
     if ("wspd" in var.name): var.name=var.name.split("_",-1)[0]
     if(i==0):
       ## initialize figure
       fig = plt.figure()
       #Get the latitude and longtitude points
       lats, lons = latlon_coords(var)
       
       #Get the basemap object
       if (bm is None or type(bm)==str):
         if(bm is "Taiwan"):
             bm = get_basemap(var,resolution='i',llcrnrlon=119.0,llcrnrlat=21.8,\
                  urcrnrlon=122.05,urcrnrlat=25.4)
         else:
             bm = get_basemap(var,resolution='i')
         bm.drawcoastlines(linewidth=0.25)
         bm.drawcountries(linewidth=0.25)
         bm.drawcounties(linewidth=1.0)
       
       #Convert Lats, lons to x and y meshgrid in numpy format for plotting usage 
       x, y= bm(lons.data, lats.data)
       
       ##Create figure and subplot
       ##determine the nrows and ncols
       if (nrows ==0):
         nrows=int(np.floor(np.sqrt(nvar)))
        
       ncols=round((nvar/nrows))
       if (ncols*nrows < nvar): 
          ncols=ncols+1
       
       cob=list()
       ax=list()
       color_range={}
       nfile_dict={str(i):nfile}
       ax.append(plt.subplot(nrows,ncols,i+1))
     else:
       nfile_dict.update({str(i):nfile})
       ax.append(plt.subplot(nrows,ncols,i+1,sharex=ax[i-1],sharey=ax[i-1]))
       
     ##Add geographic outlines
     bm.drawcoastlines(ax=ax[i])
     bm.drawcountries(ax=ax[i])

     #Add filled contour and colorbar 
     density=10
     if diff_index(diff,i):
        if (len(var)==2):
           cob.append(bm.quiver(x[::density,::density], y[::density,::density], var[0][::density,::density], var[1][::density,::density], np.sqrt(var[0]**2+var[1]**2)[::density,::density],
                     scale=30, scale_units ='width', width=0.01, edgecolor='k' , linewidths=1, ax=ax[i],cmap=get_cmap(var_cmap[i]),pivot='mid', minshaft=2))
        else:		   
           cob.append(bm.contourf(x, y,var,10,cmap=get_cmap(var_cmap[i]),ax=ax[i]))

        plt.colorbar(cob[i],shrink=.32,ax=ax[i])
     else:
       if var.name in color_range:
          if (len(var)==2):
              cob.append(bm.quiver(x[::density,::density], y[::density,::density], var[0][::density,::density], var[1][::density,::density], np.sqrt(var[0]**2+var[1]**2)[::density,::density],
                   scale=60 ,scale_units ='width', width=0.01, edgecolor='k' , linewidths=1, ax=ax[i],cmap=get_cmap(var_cmap[i]),
		   clim=(min(color_range[var.name]),max(color_range[var.name])),pivot='mid', minshaft=2))
          else:
             cob.append(bm.contourf(x, y,var,color_range[var.name], cmap=get_cmap(var_cmap[i]),ax=ax[i]))
          plt.colorbar(cob[i],shrink=.32,ax=ax[i])
       else:
          if (len(var)==2):
             cob.append(bm.quiver(x[::density,::density], y[::density,::density], var[0][::density,::density], var[1][::density,::density], np.sqrt(var[0]**2+var[1]**2)[::density,::density],
                 scale=60,scale_units ='width', width=0.01, edgecolor='k' , linewidths=1, ax=ax[i],cmap=get_cmap(var_cmap[i]),pivot='mid', minshaft=2))
          else:		 
             cob.append(bm.contourf(x, y,var,10,cmap=get_cmap(var_cmap[i]),ax=ax[i]))
          cb = plt.colorbar(cob[i],shrink=.32,ax=ax[i])
          color_range.update({var.name:cb._boundaries})

  
     #Draw title   
     var.name=var.name+" eta="+str(level)
     if diff_index(diff,i):
        var.name = var.name+" difference "
     else:
        var.name = var.name+" file"+str(nfile)

     ax[i].set_title(var.name+' at \n'+str(var.Time.data).replace("T"," ")[0:-10]+" UTC")

#     print(ax[0].title._text)
#     quit()
  if (len(nc_f1.split("/",-1))==1 or len(nc_f2.split("/",-1))==1 or mtitle==None):
     plt.suptitle(nc_f1 +'\n vs. \n'+ nc_f2)
  else:   
     plt.suptitle(nc_f1.split("/",-1)[mtitle[0]] +'\n vs. \n'+ nc_f2.split("/",-1)[mtitle[1]])
  #figManager = plt.get_current_fig_manager()
  #figManager.window.showMaximized()
  sub_linebuilders = subplotsLineBuilder(ax, bm, var_name, nc_fid1, nc_fid2, nfile_dict, diff)
  
  plt.show()
  plt.tight_layout()



def vertical_plot(lat, lon ,var_name,nc_fid1, nc_fid2, nfile, diff=False):
     if (nfile==1):
       nc_fid = nc_fid1
     elif (nfile==2):  
       nc_fid = nc_fid2

     if ("wspd" in var_name):
        var =getvar(nc_fid, var_name, units="m s-1")[0]
        if (diff is True):
           var.data = getvar(nc_fid1, var_name, units="m s-1")[0].data-getvar(nc_fid2, var_name, units="m s-1")[0].data
     else:
        var =getvar(nc_fid, var_name)
        if (diff is True):
           var.data = getvar(nc_fid1, var_name).data-getvar(nc_fid2, var_name).data

     z = getvar(nc_fid, "z")
     if ("Z"not in var.MemoryOrder):  return print("cannot plot cross section")
     start_point = CoordPair(lat=lat[0], lon=lon[0])
     end_point = CoordPair(lat=lat[1], lon=lon[1])
     var_cross = vertcross(var, z, wrfin=nc_fid, start_point=start_point,
                            end_point=end_point, latlon=True, meta=True)
     #var_cross = vertcross(var, z, start_point=start_point,
     #                       end_point=end_point, latlon=True, meta=True)
     # Create the figure
     plt.figure(figsize=(12,6))
     ax = plt.axes()

     # Make the contour plot
     var_contours = ax.contourf(var_cross, cmap=get_cmap("jet"))
     

     # Add the color bar
     plt.colorbar(var_contours, ax=ax)
     
     # Set the x-ticks to use latitude and longitude labels.
     #coord_pairs = to_np(var_cross.coords["xy_loc"])
     coord_pairs = var_cross.coords["xy_loc"].data
     x_ticks = np.arange(coord_pairs.shape[0])
     x_labels = [pair.latlon_str(fmt="{:.2f}, {:.2f}")
                 for pair in to_np(coord_pairs)]
               #  for pair in coord_pairs]
     ax.set_xticks(x_ticks[::20])
     ax.set_xticklabels(x_labels[::20], rotation=45, fontsize=8)
     
     # Set the y-ticks to be height.
     vert_vals = (var_cross.coords["vertical"].data)
     v_ticks = np.arange(vert_vals.shape[0])
     ax.set_yticks(v_ticks[::10])
     ax.set_yticklabels(vert_vals[::10], fontsize=8)
     
     # Set the x-axis and  y-axis labels
     ax.set_xlabel("Latitude, Longitude", fontsize=12)
     ax.set_ylabel("Height (m)", fontsize=12)
     
     plt.title("Vertical Cross Section of "+var_name)
     
     plt.show()

     return


class LineBuilder:

    def __init__(self, ax, bm, var_name, nc_fid1, nc_fid2, ncfile_num, index, diff):
        
        self.ax = ax
        self.bm = bm
        self.var_name = var_name
        self.nc_fid1 = nc_fid1
        self.nc_fid2 = nc_fid2
        self.ncfile_num = ncfile_num
        self.index = index
        self.diff = diff

        line, = ax.plot([float('nan')], [float('nan')])  # empty line
        self.line = line

        marker, =ax.plot([float('nan')], [float('nan')], marker="o")  # empty marker
        self.markers = marker


        self.xs = list(line.get_xdata())
        self.ys = list(line.get_ydata())
        self.cid = line.figure.canvas.mpl_connect('button_press_event', self)

        self.xs = []

        self.ys = []

    def __call__(self, event):

        if event.inaxes!=self.line.axes: return
        if (event.button==3):
          self.xs = []
          self.ys = []
          self.line.set_data(self.xs, self.ys)
          self.markers.set_data(self.xs, self.ys)

          self.line.figure.canvas.draw()

          return
        if (event.dblclick is True):
          self.xs.append(event.xdata)
          self.ys.append(event.ydata)
          self.markers.set_data(self.xs, self.ys)
          self.line.set_data(self.xs, self.ys)

          self.markers.set_color('k')

          self.line.figure.canvas.draw()
        if (len(self.xs)>=2 and event.button==2):
           print("plotting")
           lon, lat = self.bm(self.xs[-2:], self.ys[-2:], inverse=True)
           vertical_plot(lat, lon , self.var_name, self.nc_fid1, self.nc_fid2,self.ncfile_num,diff=self.diff)
           return

def subplotsLineBuilder(axes, bm, var_name, nc_fid1, nc_fid2, ncfile_dict, diff):
    sub_obj=list()
    i=0
    for ax in axes:
      sub_obj.append(LineBuilder(ax, bm, var_name[i], nc_fid1, nc_fid2, ncfile_dict[str(i)],i,diff_index(diff,i)))
      i=i+1
      
    return sub_obj 


   



import numpy as np
import matplotlib.pyplot as plt
import warnings

warnings.filterwarnings("ignore", category=RuntimeWarning)

def perform_dig(figobj, subplot = False,bais_color="red",csi_cmap="Blues"):
     grid_ticks = np.arange(0, 1.01, 0.001)
     sr_g, pod_g = np.meshgrid(grid_ticks, grid_ticks)
     bias = pod_g / sr_g
     csi = 1.0 / (1.0 / sr_g + 1.0 / pod_g - 1.0)
     if int(subplot) == 0:
         try:
             if len(figobj.axes) == 0:
                 digbg = figobj.subplots(1)
         except:
             digbg = figobj
     else:
         try:
             if len(figobj.axes) == 0:
                 digbg = figobj.subplots(1)
         except:
             digbg = figobj
     csi_contour = digbg.contourf(sr_g, pod_g, csi, np.arange(0.1, 1.1,0.1), extend="max", cmap=csi_cmap)
     b_contour = digbg.contour(sr_g, pod_g, bias, [0.1, 0.2, 0.5, 1, 2,5, 10], colors="slateblue", linestyles="dashed")
     ax2ticks = [0.1,0.2,0.5,1]
     ax2tickslabel = [1/x for x in ax2ticks]
     secax = digbg.secondary_xaxis('top')
     secay = digbg.secondary_yaxis('right')
     secax.set_xticks(ax2ticks)
     secax.set_xticklabels(ax2tickslabel, minor=False)
     plt.colorbar(csi_contour,ax=digbg)
     digbg.set_xlim(0,1)
     digbg.set_ylim(0,1)
     return digbg



import math
import numpy as np
import pandas as pd
import datetime as dt
import cwbplot

def cross2d(M, index_i, index_j, theta):
    """cross2d(2darray, array_index_i, array_index_j, math_degree)"""
    slope = math.tan(math.radians(theta))
    i_series = list()
    j_series = list()
    if abs(slope) <=1:
       for j in reversed(range(index_j)):
          i = index_i + round((j - index_j)*slope)
          if i>=M.shape[0]-1 or i<=0: break
          i_series.append(i)
          j_series.append(j)
       for j in range(index_j,M.shape[1]):
          i = index_i + round((j-index_j)*slope)
          if i>=M.shape[0]-1 or i<=0: break
          i_series.append(i)
          j_series.append(j)
    elif abs(slope)>1:
       for i in reversed(range(index_i)):
          j = index_j + round((i-index_i)/slope)
          if j>=M.shape[1]-1 or j<=0: break
          i_series.append(i)
          j_series.append(j)
       for i in range(index_i,M.shape[0]):
          j = index_j + round((i-index_i)/slope)
          if j>=M.shape[1]-1 or j<=0: break
          i_series.append(i)
          j_series.append(j)
    return i_series,j_series


def stnlatlon_stnij(data_lon, data_lat, stn_lon, stn_lat,mask=None,mask_value=0):
  """ how to use:stnlatlon_stnij(mesh_lon,mesh_lat,stn_lon, stn_lat) return index sequence 
       1D and 2D mesh grid is OK 
       mask is the array size same as data_lon and data_lat
       mask_value is the value not be calulated"""
  dist=np.sqrt((data_lat-stn_lat)**2+(data_lon-stn_lon)**2)
  if mask is None:
    indexs = np.unravel_index(np.argmin(dist), dist.shape)
    return indexs
  else:
    mask=np.array(mask)
    #surrounding_points=np.argsort(dist)[:4]
    surrounding_points=np.argsort(np.ravel(dist))[:4]
    for point in surrounding_points:
       indexs=np.unravel_index(point, dist.shape)
       if mask[indexs]!=mask_value:                   
          return indexs  
    ## if not found data in mask return the nearest point
    print('Not found in mask near',stn_lon,stn_lat,'returning nearest point')
    indexs=np.unravel_index(surrounding_points[0], dist.shape)
    return indexs     

def read_xml_cwb(data_time_str,var_list,prefix=''):
  """  read_xml_cwb(data_time_str,var_list,prefix='')
     data_time_str: string format(YYYYMMDDHHNN) in UTC
  """     
  import xmltodict as xmld
  import datetime   as dt
  import pandas as pd
  
  data_time = dt.datetime.strptime(data_time_str, "%Y%m%d%H%M")
  var_name_list=var_list.split()
  ##Reading data
  station_dict={}
  #xml_dir='/IFS6/data2/datarfs/c164/WBGT_OP/STA_DATA/'
  xml_dir='/IFS6/data2/datawrf/obsget/STA_DATA/'
  himawari_dir='/home/c164/web/'
  files=[xml_dir+data_time_str+'.QPESUMS_METRO_STATION.10M.RAD.xml',
         xml_dir+data_time_str+'.QPESUMS_AUTO_STATION.10M_RAD.xml']
  for filename in files:
    with open(filename,encoding='utf-8') as fd:
        doc =xmld.parse(fd.read())
        for j in range(len(doc['cwbopendata']['location'])):
            station_data=doc['cwbopendata']['location'][j]
            single_station_dict={}
            single_station_dict.update({'lat':float(station_data['lat_wgs84']),
                                        'lon':float(station_data['lon_wgs84'])})
            for i in range(len(station_data['weatherElement'])):
              try:
                  '''
                  if (station_data['weatherElement'][i]['elementName']=='WDIR' or
                      station_data['weatherElement'][i]['elementName']=='WDSD'):
                  '''
                  for var_name in var_name_list:
                      if (station_data['weatherElement'][i]['elementName']==var_name):
                        single_station_dict.update({prefix+station_data['weatherElement'][i]['elementName']:
                        float(station_data['weatherElement'][i]['elementValue']['value'])})
              except:
                  pass
  
            if all(x>-99 for x in single_station_dict.values() ):
               station_dict.update({station_data['stationId']:single_station_dict})
  ###convert to dictionary format
  data= pd.DataFrame.from_dict(station_dict,orient='index')
  return data

def stnlatlon_stnij_WGS84(data_lon, data_lat, stn_lon, stn_lat,mask=None,mask_value=0):
  """ how to use:stnlatlon_stnij(mesh_lon,mesh_lat,stn_lon, stn_lat) return index sequence
       1D and 2D mesh grid is OK
       mask is the array size same as data_lon and data_lat
       mask_value is the value not be calulated"""

  import pyproj
  g = pyproj.Geod(ellps='WGS84')
  dist=np.sqrt((data_lat-stn_lat)**2+(data_lon-stn_lon)**2)
  ## finding nearset 4four point index in fatten index  
  nearby_points=np.argsort(np.ravel(dist))[:4]
  
  ## finding ij index of points  (nearby_index)  
  nearby_indexs=(np.unravel_index(nearby_points,dist.shape))

  ## getting data array for corresponding points  
  mesh_stn_lon=np.full(nearby_points.shape,stn_lon)  
  mesh_stn_lat=np.full(nearby_points.shape,stn_lat)  
  ## calculate distance using greate circle distance  
  if type(data_lon).__name__ == 'DataArray':
      _,_,gdist=g.inv(data_lon.data[nearby_indexs],data_lat.data[nearby_indexs] \
                     ,mesh_stn_lon,mesh_stn_lat)
  elif type(data_lon).__name__ == 'ndarray':
     _,_,gdist=g.inv(data_lon[nearby_indexs],data_lat[nearby_indexs] \
                     ,mesh_stn_lon,mesh_stn_lat)
  gdist=np.array(gdist)

  ## resorting the great circle distance(shape=4), returning index 
  g_points=np.argsort(gdist)

  if mask is None:  
    index = np.unravel_index(nearby_points[g_points[0]], data_lon.shape)
    return index
  else:
    mask=np.array(mask)
    for point in g_points:
       index=np.unravel_index(nearby_points[point], data_lon.shape)
 
       if mask[index]!=mask_value:
          return index
    ## if not found data in mask return the nearest point
    print('Not found in mask near',stn_lon,stn_lat,'returning nearest point')
    min_indexs=np.unravel_index(nearby_points[g_points[0]],data_lon.shape)
    return index
  
def read_nc_with_latlonpd(data_time_str,df,nc_path,var_list,prefix='Model',layer=0):
    ''' pandas colums require lon lat info 
    '''
    import netCDF4 as nc
    import datetime as dt
    from wrf import getvar
    if  isinstance(data_time_str,dt.datetime):
        print('detect datetime format')
        data_time=data_time_str
    else:    
        print('detect string format')
        data_time = dt.datetime.strptime(data_time_str, "%Y%m%d%H%M")
    f_name=dt.datetime.strftime(data_time, "wrfout_d02_%Y-%-m-%d_%H:%M:%S")
    print('reading IO')
    nc_fid=nc.Dataset(nc_path+f_name,'r')
    print('end reading IO')
    var_name =var_list.split()
    for var in var_name:
        try:
            if ("Z" in getvar(nc_fid,var).MemoryOrder):
               data=getvar(nc_fid,var)[layer].data
            else:
                data=getvar(nc_fid,var).data
        except:
             pass
     #### convert model to obs format       
        if var=='WDSD':     data=getvar(nc_fid,'uvmet10_wspd_wdir')[0].data
        if var=='WDIR':     data=getvar(nc_fid,'uvmet10_wspd_wdir')[1].data  
        if var=='TEMP':     data=getvar(nc_fid,'T2')[:].data
        if var=='HUMID':    data=getvar(nc_fid,'rh2')[:].data         
        if var=='DD_GRM10': data=getvar(nc_fid,'SWDOWN')[:].data      
        if var=='ELEV':     data=getvar(nc_fid,'ter')[:].data  
        if var=='PRES':    data=getvar(nc_fid,'pressure')[0].data   
            
        if not prefix+'ij' in df.columns:
           #AAdf[prefix+'ij']=df[['lon','lat']].apply(lambda x: tools.stnlatlon_stnij_WGS84 \
           #AA     (data.XLONG,data.XLAT,x[0],x[1],mask=getvar(nc_fid,'XLAND')[:]),axis=1)
           df[prefix+'ij']=df[['lon','lat']].apply(lambda x: stnlatlon_stnij_WGS84 \
                (getvar(nc_fid,'XLONG'),getvar(nc_fid,'XLAT'),\
                 x[0],x[1],mask=getvar(nc_fid,'XLAND')[:]),axis=1)
           #df[prefix+'_ij']=df[['lon','lat']].apply(lambda x: stnlatlon_stnij \
           #     (data.XLONG,data.XLAT,x[0],x[1]),axis=1)
        #AAdf[prefix+var]=df[prefix+'ij'].apply(lambda row: data.d ata[row])
        df[prefix+var]=df[prefix+'ij'].apply(lambda row: data[row])
    return df 

def DMSXYDIM(WXX):
    ''' function for return xdim,ydim at once
        EX: xdim,ydim=cwbplot.tools.DMSXYDIM("WF02")
    '''
    if WXX=='WD01': xdim,ydim=661,385
    if WXX=='WD02': xdim,ydim=1158,673
    if WXX=='WF02': xdim,ydim=263,303
    if WXX=='WE01': xdim,ydim=450,450
    return xdim,ydim

def DMSTAIL(WXX):
    ''' function for return tail of DMSKEY in string format
    '''
    domain2grid={"WD01":"0254485" ,"WD02":"0779334" ,"WF02":"0079689","WD03":"0729136","WE01":"0202500"}
    return domain2grid[WXX]

def DMSLONLAT(WXX,reshape=False, order="F"):
    ''' function for return DMSKEY LON/LAT array at once
        if reshape=True return (xdim,ydim) array)
        EX: dms_lon, dms_lat = cwbplot.tools.DMSLONLAT("WF02",reshape=True)
    '''
    #dmslatlon_path='/home/c052/Pub/anaconda3/envs/plotenv/lib/python3.6/site-packages/cwbplot/sharedata/'
    dmslatlon_path= cwbplot.__path__[0] + "/sharedata/"
    domain2grid={"WD01":"0254485" ,"WD02":"0779334" ,"WF02":"0079689","WD03":"0729136","WE01":"0202500"}
    xdim,ydim = DMSXYDIM(WXX)


    dms_lat   = np.fromfile(dmslatlon_path+'X00LAT'+WXX+'H'+DMSTAIL(WXX),dtype='>d',count=-1,sep="") #LAT
    dms_lon   = np.fromfile(dmslatlon_path+'X00LON'+WXX+'H'+DMSTAIL(WXX),dtype='>d',count=-1,sep="") #LON

    if reshape==True:
       if order == "F":
           data_lat = np.reshape(dms_lat, (xdim,ydim), order=order)
           data_lon = np.reshape(dms_lon, (xdim,ydim), order=order)
       elif order == "C":
           data_lat = np.reshape(dms_lat, (ydim,xdim), order=order)
           data_lon = np.reshape(dms_lon, (ydim,xdim), order=order)
       return data_lon,data_lat
    else:
       return dms_lon,dms_lat
def DMSRESHAPE(var,WXX):
    ''' function for reshape data
        EX: reshape_var = cwbplot.tools.DMSRESHAPE(var,'WF02')
    '''
    xdim,ydim = DMSXYDIM(WXX)
    reshape_var = np.reshape(var, (xdim,ydim), order='F')
    return reshape_var
def Caldtg(dtg,hr):
    if type(dtg).__name__ == 'datetime':
       data_time=dtg
       return dtg+dt.timedelta(hours=hr)
    else:
       try:
         data_time = dt.datetime.strptime(str(dtg),"%Y%m%d%H")+dt.timedelta(hours=hr)
         dt_format="%Y%m%d%H"
       except: 
         data_time = dt.datetime.strptime(str(dtg),"%y%m%d%H")+dt.timedelta(hours=hr)
         dt_format="%y%m%d%H"
       return dt.datetime.strftime(data_time,dt_format) 



def GETPROJ(model, domain, lon=False, lat=False, res='l',dms=False):
    from mpl_toolkits.basemap import Basemap
    WRFM04d01=[ 120.0, 27.065534591674805, 10.0, 40.0]
    WRFM04d02=[ 120.0, 27.065534591674805, 10.0, 40.0]
    WRFM05d01=[ 120.0, 27.065534591674805, 10.0, 40.0]
    WRFM05d02=[ 120.0, 27.065534591674805, 10.0, 40.0]
    RWRFM01d01=[ 120.0, 21.494176864624023, 10.0, 40.0]
    lcc = pd.DataFrame([WRFM04d01,WRFM04d02,WRFM05d01,WRFM05d02,RWRFM01d01],\
                    columns =["lon0","lat0","lat1","lat2"],\
                    index=["wrfm04wd01","wrfm04wd02","wrfm05wd01","wrfm05wd02","rwrfm01we01"])
    cwbplotpath = cwbplot.__path__[0]
    idx = model.lower() + domain.lower()
    lon0, lat0 = lcc.loc[idx]["lon0"], lcc.loc[idx]["lat0"]
    lat1, lat2 = lcc.loc[idx]["lat1"], lcc.loc[idx]["lat2"]
    if dms:
        if idx == "wrfm04wd01":
            lon, lat = np.load(f"{cwbplotpath}/lonlatinfo/WRFM04D01LONLAT.npy")
        elif idx == "wrfm04wd02" or idx == "wrfm04wf02":
            lon, lat = np.load(f"{cwbplotpath}/lonlatinfo/WRFM04D02LONLAT.npy")
        elif idx == "rwrfm01we01":
            lon, lat = np.load(f"{cwbplotpath}/lonlatinfo/RWRFLONLAT.npy")
        if idx == "wrfm04wf02":
            idx = "wrfm04wd02"
        wrfproj = Basemap(projection="lcc", resolution = res, rsphere = 6370000., \
            lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2, \
            llcrnrlat = lat[0,0], \
            llcrnrlon = lon[0,0], \
            urcrnrlat = lat[-1,-1], \
            urcrnrlon = lon[-1,-1])
    else:
        if lon and lat:
            wrfproj = Basemap(projection="lcc", resolution = res, rsphere = 6370000., \
               lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2, \
               llcrnrlat = lat[0,0], \
               llcrnrlon = lon[0,0], \
               urcrnrlat = lat[-1,-1], \
               urcrnrlon = lon[-1,-1])
        else:
            print("Not dms format file, need lon and lat info")
    return wrfproj
   
                                                                



from mpl_toolkits.basemap import Basemap
import pandas as pd
import cwbplot

WRFM04d01=[ 120.0, 27.065534591674805, 10.0, 40.0]
WRFM04d02=[ 120.0, 27.065534591674805, 10.0, 40.0]
WRFM05d01=[ 120.0, 27.065534591674805, 10.0, 40.0]
WRFM05d02=[ 120.0, 27.065534591674805, 10.0, 40.0]
RWRFM01d01=[ 120.0, 21.494176864624023, 10.0, 40.0]


lcc = pd.DataFrame([WRFM04d01,WRFM04d02,WRFM05d01,WRFM05d02,RWRFM01d01],\
                    columns =["lon0","lat0","lat1","lat2"],\
                    index=["WRFM04d01","WRFM04d02","WRFM05d01","WRFM05d02","RWRFM01d01"])

def wrfd(lon, lat,  domain, projection= 'lcc', res="l", cut = False, center =False, ax=False):
    idx = "WRFM04"+domain
    lon0, lat0 = lcc.loc[idx]["lon0"], lcc.loc[idx]["lat0"]
    lat1, lat2 = lcc.loc[idx]["lat1"], lcc.loc[idx]["lat2"]
    if cut == False and projection == 'lcc' and center == False:
        if ax == False:
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2,
                    llcrnrlat = lat[0,0],
                    llcrnrlon = lon[0,0],
                    urcrnrlat = lat[-1,-1],
                    urcrnrlon = lon[-1,-1])
        else:
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2,
                    llcrnrlat = lat[0,0],
                    llcrnrlon = lon[0,0],
                    urcrnrlat = lat[-1,-1],
                    urcrnrlon = lon[-1,-1], ax = ax)
    elif isinstance(cut,tuple) or isinstance(cut,list) and projection == "lcc" and center==False:
        if len(cut) == 4:
            if ax == False:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1])
            else:
                 wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1], ax= ax)
        else:
            print("Object cut length must 4")
    elif not isinstance(cut,tuple) and not isinstance(cut,list) and projection == "lcc" and center==False:
        print("Object cut type must tuple or list")
    
    if cut == False and projection == 'cyl':
        if center == False:
            if ax == False:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0,
                    llcrnrlat = lat.min(),
                    llcrnrlon = lon.min(),
                    urcrnrlat = lat.max(),
                    urcrnrlon = lon.max())
            else:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0,
                    llcrnrlat = lat.min(),
                    llcrnrlon = lon.min(),
                    urcrnrlat = lat.max(),
                    urcrnrlon = lon.max(), ax = ax)
        if isinstance(center,tuple) or isinstance(center,list):
            if ax == False:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = center[0], lat_0 = center[1])
            else:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = center[0], lat_0 = center[1], ax= ax)
        elif isinstance(center,tuple) and isinstance(center,list):
            print("Object center must tuple or list and length must 2")
    elif isinstance(cut,tuple) or isinstance(cut,list) and projection == "cyl":
        if center == False:
            if ax == False:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 =lat0 , 
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1])
            else:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 =lat0 ,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1], ax= ax)
        if isinstance(center,tuple) or isinstance(center,list):
            if ax == False:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 =lat0 ,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1])
            else:
                wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 =lat0 ,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1], ax= ax)
        elif isinstance(center,tuple) and isinstance(center,list):
            print("Object center must tuple or list and length must 2")
    elif cut and projection == "cyl":
        if isinstance(cut,tuple) and isinstance(cut,list):
            print("Object cut type must tuple or list")
    
    return wrfproj


def rwrf(lon, lat,  domain="d01", projection= 'lcc', res="l", cut = False, center =False):
    idx = "RWRFM01"+domain
    lon0, lat0 = forlcc.loc[idx]["lon0"], forlcc.loc[idx]["lat0"]
    lat1, lat2 = forlcc.loc[idx]["lat1"], forlcc.loc[idx]["lat2"]
    if cut == False and projection == 'lcc' and center == False:
        wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2,
                    llcrnrlat = lat[0,0],
                    llcrnrlon = lon[0,0],
                    urcrnrlat = lat[-1,-1],
                    urcrnrlon = lon[-1,-1])
    elif isinstance(cut,tuple) or isinstance(cut,list) and projection == "lcc" and center==False:
        if len(cut) == 4:
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0, lat_1 = lat1, lat_2 = lat2,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1])
        else:
            print("Object cut length must 4")
    elif not isinstance(cut,tuple) and not isinstance(cut,list) and projection == "lcc" and center==False:
        print("Object cut type must tuple or list")
    
    if cut == False and projection == 'cyl':
        if center == False:
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 = lat0,
                    llcrnrlat = lat.min(),
                    llcrnrlon = lon.min(),
                    urcrnrlat = lat.max(),
                    urcrnrlon = lon.max())
        if isinstance(center,tuple) or isinstance(center,list):
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = center[0], lat_0 = center[1])
        elif isinstance(center,tuple) and isinstance(center,list):
            print("Object center must tuple or list and length must 2")
    elif isinstance(cut,tuple) or isinstance(cut,list) and projection == "cyl":
        if center == False:
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 =lat0 ,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1])
        if isinstance(center,tuple) or isinstance(center,list):
            wrfproj = Basemap(projection=projection, resolution = res, rsphere = 6370000. ,
                    lon_0 = lon0, lat_0 =lat0 ,
                    llcrnrlat = cut[2],
                    llcrnrlon = cut[0],
                    urcrnrlat = cut[3],
                    urcrnrlon = cut[1])
        elif isinstance(center,tuple) and isinstance(center,list):
            print("Object center must tuple or list and length must 2")
    elif cut and projection == "cyl":
        if isinstance(cut,tuple) and isinstance(cut,list):
            print("Object cut type must tuple or list")
    return wrfproj



import matplotlib.pyplot as plt
import datetime as dt
import numpy as np
import re

###reading GRADS data
#grads_dir='/IFS6/data2/datawrf/p108/QPESUMS/data/QPE_2D_2019012812.dat'
compress_dir='/IFS6/data2/datawrf/p108/QPESUMS/rerun_casedata/raincase_o/COMPREF.20181115.0700.gz'
#compress_dir='/IFS6/data2/datarfs/c164/test_data/radar/MREF/MREF3D30L.20200212.0100'
#def QPE_2D_READ(fname):
class QPE_2D_READ:
  def __init__(self,fname):
    data=np.fromfile(fname,dtype='>f',count=-1,sep="")
    self.time_str=re.split('QPE_2D_|.dat',fname)[-2]
    self.datetime=dt.datetime.strptime(self.time_str,"%Y%m%d%H")
    self.nx=441
    self.ny=561
    self.ll_lon=118
    self.ll_lat=20
    self.grid_step=0.0125
    self.undef=-999.
    self.lon_1d = self.ll_lon+np.arange(0,self.nx)*self.grid_step
    self.lat_1d = self.ll_lat+np.arange(0,self.ny)*self.grid_step
    self.lat, self.lon= np.meshgrid(self.lat_1d,self.lon_1d)
    self.data=np.float32(np.reshape(data, (self.nx,self.ny), order='F'))
    #data[data<=0]=np.NaN
  def __str__(self):
     return "CWB QPE_2D data binaray format("+str(np.dtype('>f').descr)+")\n" \
            "TIME:"+ self.datetime.ctime()+"\n" \
            "DIMX,DIMY="+str(self.nx)+','+str(self.ny)+"\n" 
  def __call__(self):
      return self.data
  @classmethod  
  def cwbcolorbar(self):  
    from matplotlib.colors import ListedColormap
    from matplotlib.colors import BoundaryNorm
    rain_color  =["#fdfdfd","#ccccb3","#99ffff","#66b3ff","#0073e6","#002699",\
                  "#009900","#1aff1a","#ffff00","#ffcc00","#ff9900","#ff0000",\
                  "#cc0000","#990000","#800040","#b30047","#ff0066","#ff80b3"]
    rain_colormap = ListedColormap(rain_color)
    clevel=[0,1,2,6,10,15,20,30,40,50,60,70,80,90,110,130,150,200]
    norm=BoundaryNorm(clevel,len(clevel))
    dict({'levels':clevel,'norm':norm,'cmap':rain_colormap})
     
    return  dict({'levels':clevel,'norm':norm,'cmap':rain_colormap})
class COMPREF_READ:
  def __init__(self,fname):
    import struct
    if '.gz' in fname:
        import gzip
        print('unzip files')
        fid = gzip.open(fname).read()
    else:
        fid = open(fname,"rb").read()
    ## Reading time info    
    info=np.array(struct.unpack('<9i4s10i',fid[0:80]))
    self.yyyy=int(info[0])
    self.mm=  int(info[1])
    self.dd=  int(info[2])
    self.hh=  int(info[3])
    self.mn=  int(info[4])
    self.ss=  int(info[5])
    self.nx=  int(info[6])
    self.ny=  int(info[7])
    self.nz=  int(info[8])
    self.time_str=str(self.yyyy)+str(self.mm).zfill(2)+str(self.dd).zfill(2)+str(self.hh).zfill(2)\
                       +str(self.mn).zfill(2)+str(self.ss).zfill(2)
    self.datetime=dt.datetime.strptime(self.time_str,"%Y%m%d%H%M%S")
    ## Reading map projection and grid scale                   
    self.proj=info[9]
    self.map_scale= int(info[10])
    self.projlat1 = int(info[11])
    self.projlat2 = int(info[12])
    self.projlon  = int(info[13])
    self.alon     = int(info[14])
    self.alat     = int(info[15])
    self.xy_scale = int(info[16])
    self.dx       = int(info[17])
    self.dy       = int(info[18])
    self.dxy_scale= int(info[19])
    info=np.array(struct.unpack('<'+str(self.nz+11)+'i20s6s3i',fid[80:80+(self.nz+14)*4+26]))
    next_ind=80+self.nz*4
    self.zht=np.array(struct.unpack('<'+str(self.nz)+'i',fid[80:next_ind])).astype(np.float32)
    info=np.array(struct.unpack('<11i20s6s3i',fid[next_ind:next_ind+14*4+26]))
    self.z_scale=int(info[0])
    self.i_bb_mode=int(info[1])
    self.unkn01=np.array(info[2:10]).astype(np.float32)
    ## Reading data
    self.varname=str(info[11],'utf-8','ignore').replace('\x00','')
    self.varunit=str(info[12],'utf-8','ignore').replace('\x00','')
    self.var_scale= int(info[13])
    self.missing  = int(info[14])
    self.nradar   = int(info[15])
    next_ind=next_ind+14*4+26
    self.mosradar=re.findall('....',(str(struct.unpack('<'+str(self.nradar*4)+'s', \
                  fid[next_ind:next_ind+self.nradar*4])[0],'utf-8','ignore')))
    data_int = struct.unpack(self.nx*self.ny*self.nz*'h', fid[-self.nx*self.ny*self.nz*2:])
    data=np.array(data_int).astype(np.float32)/self.var_scale

    if '30L' in fname:
      self.file_type='mosaicked refl RWRF'
      self.lon, self.lat = np.load('/home/c052/Pub/anaconda3/envs/plotenv/lib/python3.6/site-packages/cwbplot/sharedata/MREF_lonlat.npy')
    else:  
      if 'CREF' in self.varname:
         self.file_type='CREF'                                       
      elif 'CB' in self.varname:
         self.file_type='QPE'                                       
      elif 'mosaicked refl' in self.varname:
         self.file_type='mosaicked refl'
                                                                    ## (alon,alat)     (alon+dlon,alat)  
      self.ll_lon=self.alon/self.xy_scale                           ##      +---------------+
      self.ur_lat=self.alat/self.xy_scale                           ##      |               |
      self.ur_lon=self.ll_lon+self.dx/self.dxy_scale*(self.nx-1)    ##      |               |  alon=dy/dxy*(ny-1)
      self.ll_lat=self.ur_lat-self.dy/self.dxy_scale*(self.ny-1)    ##      |               |  alat=dx/dxy*(nx-1)
      self.lon_1d = np.linspace(self.ll_lon, self.ur_lon, self.nx)  ##      |               |
      self.lat_1d = np.linspace(self.ll_lat, self.ur_lat, self.ny)  ##      |               |
      self.lat, self.lon= np.meshgrid(self.lat_1d,self.lon_1d)      ##      +---------------+
                                                                    ## (alon,alt-dlat)  (alon+dlon,alat-dlat)   
    if self.nz>1 :
       self.data=(np.reshape(data, (self.nx,self.ny,self.nz), order='F'))
    else:   
       self.data=(np.reshape(data, (self.nx,self.ny), order='F'))

  def __str__(self):
     return "CWB "+self.file_type+" binary format("+str(np.dtype('h').descr)+")\n" \
            "TIME:"+ self.datetime.ctime()+"\n" \
            "DIMX,DIMY,DIMZ="+str(self.nx)+','+str(self.ny)+','+str(self.nz)+"\n" 
  def __call__(self):
      return self.data
  def cwbcolorbar(self):  
    from matplotlib.colors import ListedColormap
    from matplotlib.colors import BoundaryNorm
    if  any( x in self.file_type for x in ["REF","ref"]):
      radar_color =["#84C1FF","#2894FF","#0066CC","#28FF28","#00BB00","#009100",\
                    "#F9F900","#FF8000","#FF5151","#EA0000","#AE0000","#FF44FF",\
                    "#D200D2","#750075"]
      radar_colormap = ListedColormap(radar_color)
      clevel =[5,10,15,20,25,30,35,40,45,50,55,60,65,70,75]
      norm=BoundaryNorm(clevel,len(clevel))
      dict({'levels':clevel,'norm':norm,'cmap':radar_colormap})
       
      return  dict({'levels':clevel,'norm':norm,'cmap':radar_colormap})
    
    elif 'QPE' in self.file_type:
      rain_color  =["#fdfdfd","#ccccb3","#99ffff","#66b3ff","#0073e6","#002699",\
                    "#009900","#1aff1a","#ffff00","#ffcc00","#ff9900","#ff0000",\
                    "#cc0000","#990000","#800040","#b30047","#ff0066","#ff80b3"]
      rain_colormap = ListedColormap(rain_color)
      clevel=[0,1,2,6,10,15,20,30,40,50,60,70,80,90,110,130,150,200]
      norm=BoundaryNorm(clevel,len(clevel))
      dict({'levels':clevel,'norm':norm,'cmap':rain_colormap})
       
      return  dict({'levels':clevel,'norm':norm,'cmap':rain_colormap})
class DOPPLER_3D_READ:
  def __init__(self,fname):
    import struct
    if '.gz' in fname:
        import gzip
        print('unzip files')
        fid = gzip.open(fname).read()
    else:
        fid = open(fname,"rb").read()
    
    self.nx=441
    self.ny=561
    self.nz=10
    self.ll_lon=118
    self.ll_lat=20
    self.grid_step=0.0125
    self.undef=-99.
    self.lon_1d = self.ll_lon+np.arange(0,self.nx)*self.grid_step
    self.lat_1d = self.ll_lat+np.arange(0,self.ny)*self.grid_step
    self.lat, self.lon= np.meshgrid(self.lat_1d,self.lon_1d)
    record = np.array(struct.unpack('<4948060f', fid))  ### Every fortran unformatted data has 4 bytes header and tailing
    index = np.where(abs(record) <=1.38673057e-39)        ### The header and tailing encode to float32 are small number 10^(-39)
    data = np.delete(record, index)
    data[np.where(data==self.undef)] = float('nan')
    data_3d = np.reshape(data,(441,561,-1), order='F')
    self.data_u = data_3d[:,:,::2]
    self.data_v = data_3d[:,:,1::2]
class MSC_READ:
   def __init__(self,fname):
     import numpy as np
     self.data=dict()
     gispath='/home/c052/Pub/anaconda3/envs/plotenv/lib/python3.6/site-packages/cwbplot/sharedata/'
     file_proj={1350:gispath+'Proj_Python_Scale005_1350x1350',
                2750:gispath+'Proj_Python_Scale050_2750x2750'}
     data_type={'ALB':'f','CNT':'uint8','BTP':'f'}
     ### ingest data
     if '.gz' in fname:
         self.res=int(fname.split('.')[-3])
         self.band=fname.split('.')[-4]
         self.area=fname.split('.')[-5]   
         import tarfile
         file=tarfile.open(fname, "r:gz")   
         for member in file:
           print(member)
           var_name=member.name.split('.')[-1].upper()
           array=np.frombuffer(file.extractfile(member).read(),dtype=data_type[var_name]) 
           self.data[self.band+'_'+var_name] =  np.reshape(array,(self.res,self.res), order='C')

     else:       
         self.res=int(fname.split('.')[-2])  
         self.band=fname.split('.')[-3]
         self.area=fname.split('.')[-4]    
         var_name=fname.split('.')[-1].upper()
         array = np.fromfile(fname,dtype=data_type[var_name],count=-1,sep="")
         self.data[self.band+'_'+var_name] =  np.reshape(array,(self.res,self.res), order='C')
    ## Ingest projection info
     array_proj=np.fromfile(file_proj[self.res],dtype='f',count=-1,sep="")
     lat,lon =array_proj[0:self.res**2],array_proj[-self.res**2:]
     self.lon = np.reshape(lon, (self.res,self.res), order='C')
     self.lat = np.reshape(lat, (self.res,self.res), order='C')   
    



import xml.etree.ElementTree as ET
import pandas as pd
import numpy as np
import json
import pygrib as pb
import matplotlib.pyplot as plt
from datetime import datetime
from cwbplot.json import *
from cwbplot.obsvis import *
from cwbplot import cwb_colorbar
from mpl_toolkits.basemap import Basemap

try:
    from osgeo import gdal
except:
    print("Gdal package can't be imported, which probably not be installed")
    print("Function for read geotiff data can't be used")



def judecat(idname):
    if "_" in idname:
        splitidx = idname.split("_")
        if "CWB" in splitidx:
            splitidx.remove("CWB")
    elif "-" in idname:
        splitidx = idname.split("-")
    lengstr = [ len(xx) for xx in splitidx ]
    if len(lengstr) <= 1:
        idxmainstr = splitidx[lengstr.index(max(lengstr))]
        idxsubstr = False
    elif len(lengstr) > 1:
        idxmainstr = splitidx[lengstr.index(max(lengstr))]
        idxsubstr = splitidx[lengstr.index(3)]
    return idxmainstr, idxsubstr

def onestn(lociter, elements):
    """
    lociter代表針對根的location的iter，即roots.iter("{urn:cwb:gov:tw:cwbcommon:0.1}location")
    以此份資料來說，一個location代表一個測站。
    elements代表location的該層資訊及下一層標記。
    """
    valu, header = [],[]  #儲存數值及檔頭用
    cnt = 1
    itercol = [ lociter.iter(eachtag) for eachtag in elements] #先對每個element進行iter
    
    for eachiter, eachelement in zip(itercol,elements):
        for elements in eachiter:
            if "weatherElement" not in eachelement and "parameter" not in eachelement and "time" not in eachelement:
                header.append(eachelement.split("}")[-1])
                valu.append(elements.text)
            else:
                for qqinin in elements:
                    if  "weatherElement" in eachelement:
                        for xxin in qqinin.iter("{urn:cwb:gov:tw:cwbcommon:0.1}elementName"):
                            if cnt == 1:                            
                                header.append(xxin.text)
                        for xxin in qqinin.iter("{urn:cwb:gov:tw:cwbcommon:0.1}elementValue"):
                            valu.append(xxin[0].text)
                    elif "parameter" in eachelement:
                        for xxin in qqinin.iter("{urn:cwb:gov:tw:cwbcommon:0.1}parameterName"):
                            if cnt ==1:
                                header.append(xxin.text)
                        for xxin in qqinin.iter("{urn:cwb:gov:tw:cwbcommon:0.1}parameterValue"):
                            valu.append(xxin.text)
                    else:
                        header.append(eachelement.split("}")[-1])
                        obst = datetime.strptime(qqinin.text,"%Y-%m-%dT%H:%M:%S+08:00").strftime("%Y%m%d%H")
                        valu.append(obst)
    cnt = 2
    return header, valu

def radarcomp(fn_roots,catgo):
    if catgo == "xml":
        for qq in fn_roots.iter("{urn:cwb:gov:tw:cwbcommon:0.1}dataset"):
            for qqqq in qq:
                if "contents" in qqqq.tag.split("}"):
                    rdecho = qqqq[1].text.split(",")
                    floatradar = [ float(xx) for xx in rdecho]
        for qq in fn_roots.iter("{urn:cwb:gov:tw:cwbcommon:0.1}parameter"):
            if qq[0].text == "左下角":
                lonstr, latstr = qq[1].text.split(",")
                startlon, startlat = float(lonstr), float(latstr)
            if qq[0].text == "時間":
                obstime = datetime.strptime(qq[1].text,"%Y-%m-%dT%H:%M:%S+08:00").strftime("%Y%m%d%H")
            if qq[0].text == "解析度":
                resolution = float(qq[1].text)
            if qq[0].text == "維度(nx*ny)":
                xgridstr, ygridstr = qq[1].text.split("*")
                xgrid, ygrid = int(xgridstr), int(ygridstr)
    elif catgo == "json":
        rdecho = fn_roots["cwbopendata"]["dataset"]["contents"]["content"].split(",")
        floatradar = [ float(xx) for xx in rdecho]
        for qq in fn_roots["cwbopendata"]["dataset"]["datasetInfo"]["parameterSet"]["parameter"]:
            if qq["parameterName"] == "左下角":
                lonstr, latstr = list(qq.values())[1].split(",")
                startlon, startlat = float(lonstr), float(latstr)
            if qq["parameterName"] == "時間":
                obstime = datetime.strptime(list(qq.values())[1],"%Y-%m-%dT%H:%M:%S+08:00").strftime("%Y%m%d%H")
            if qq["parameterName"] == "解析度":
                resolution = float(list(qq.values())[1])
            if qq["parameterName"] == "維度(nx*ny)":
                xgridstr, ygridstr = list(qq.values())[1].split("*")
                xgrid, ygrid = int(xgridstr), int(ygridstr)
    lonx = np.linspace(startlon, startlon+resolution*(xgrid-1), xgrid)
    laty = np.linspace(startlat, startlat+resolution*(ygrid-1), ygrid)
    lon, lat = np.meshgrid(lonx, laty)
    radar = np.array(floatradar).reshape(ygrid,xgrid)
    return radar, lon, lat, obstime

def gtiff(fn,infolatlon):
    sattif = gdal.Open(fn,gdal.GA_ReadOnly)
    data  = sattif.GetRasterBand(1) #取值
    data2arr = data.ReadAsArray() #僵值轉換為array
    albdo = data2arr/1000.
    llx, pixelx, skx, lly, sky, pixely = sattif.GetGeoTransform()
    lrx = llx + (sattif.RasterXSize * pixelx)
    lry = lly + (sattif.RasterYSize* pixely)
    return albdo, infolatlon

def read_xml(fn,tiff_fn=False):
    trees = ET.parse(fn)
    roots = trees.getroot()
    ids = next(roots.iter("{urn:cwb:gov:tw:cwbcommon:0.1}dataid")).text
    maincat, subcat = judecat(ids)
    if maincat == "A0001" or maincat=="A0002" or maincat == "A0003":
        loc_element = ['{urn:cwb:gov:tw:cwbcommon:0.1}time',
               '{urn:cwb:gov:tw:cwbcommon:0.1}stationId',
               '{urn:cwb:gov:tw:cwbcommon:0.1}locationName',
               '{urn:cwb:gov:tw:cwbcommon:0.1}lon',
               '{urn:cwb:gov:tw:cwbcommon:0.1}lat',
               '{urn:cwb:gov:tw:cwbcommon:0.1}lon_wgs84',
               '{urn:cwb:gov:tw:cwbcommon:0.1}lat_wgs84',
               '{urn:cwb:gov:tw:cwbcommon:0.1}weatherElement',
               '{urn:cwb:gov:tw:cwbcommon:0.1}parameter',
              ]
        packvalue = []
        for qq in roots.iter("{urn:cwb:gov:tw:cwbcommon:0.1}location"):
            ch, eachvalue = onestn(qq, loc_element)
            packvalue.append(eachvalue)
        xml2df = pd.DataFrame(packvalue,columns=ch)
        dfORarr = xml2df       
    elif maincat == "A0059":
        radararr = radarcomp(roots, catgo="xml")
        dfORarr = radararr
    elif maincat == "B0056":
        if tiff_fn == False:
            print("Plz give geotiff filename with path")
        else:
            lonlatlist = []
            for lonlat in roots.iter('{urn:cwb:gov:tw:cwbcommon:0.1}parameterValue'):
                limt_l, limt_r = lonlat.text.split(" - ")
                lonlatlist.append(float(limt_l))
                lonlatlist.append(float(limt_r))
            sat = gtiff(tiff_fn, lonlatlist)
            dfORarr = sat
    return dfORarr

def read_json(fn):
    with open(fn,"r",encoding='utf-8') as jj:
        fn = json.load(jj)
    ids = fn["cwbopendata"]["dataid"]
    maincat, subcat = judecat(ids)
    if maincat == "A0001" or maincat == "A0002" or maincat =="A0003":
        collout = []
        for stn in range(len(fn["cwbopendata"]["location"])):
            dicts =  fn["cwbopendata"]["location"][0].keys()
            coll = []
            header = []
            for dic in dicts:        
                fndict = fn["cwbopendata"]["location"][stn][dic]
                if dic == "time":
                    header.append(dic)
                    timedict = fndict['obsTime']
                    coll.append(datetime.strptime(timedict,"%Y-%m-%dT%H:%M:%S+08:00").strftime("%Y%m%d%H"))
                elif dic != "weatherElement" and dic != "parameter" and dic != "time":
                    header.append(dic)
                    coll.append(fndict)
                elif dic == "weatherElement":
                    for element in range(len(fndict)):
                        header.append(fndict[element]["elementName"])
                        coll.append(fndict[element]["elementValue"]["value"])
                elif dic == "parameter":
                    for parame in range(len(fndict)):
                        header.append(fndict[parame]["parameterName"])
                        coll.append(fndict[parame]["parameterValue"])
            collout.append(coll)
            df = pd.DataFrame(collout, columns = header)
            header.remove("time")
            header.insert(0,"time")
            finaldf = df.loc[:,header]
            dfORarr = finaldf
    elif maincat == "A0059":
        radararr = radarcomp(fn, catgo="json")
        dfORarr = radararr
    return dfORarr

def draw(fn=False,types=False, apipath=False, cut=False, metvars=False):
    if fn and not apipath:
        with open(fn,"r",encoding='utf-8') as jj:
            fn = json.load(jj)
        ids = fn["cwbopendata"]["dataid"]
        maincat, subcat = judecat(ids)
    else:
        maincat = False
    if maincat == "A0059":
        fig = plt.figure()
        radar, lon, lat, obstime = radarcomp(fn, catgo="json")
        radar[radar == -99.] = np.nan
        radarcm = cwb_colorbar.radar()
        projs = Basemap(projection='merc',resolution="l",urcrnrlat=lat[-1,-1], llcrnrlat=lat[0,0], llcrnrlon=lon[0,0], urcrnrlon=lon[-1,-1])
        projs.drawcoastlines()
        if types == "scatter":
            sct = projs.scatter(lon, lat, c=radar, cmap = radarcm["cmap"], norm=radarcm["norm"], latlon=True)
        elif types == "imshow":
            sct = projs.imshow(radar,cmap = radarcm["cmap"], norm=radarcm["norm"],extent=(lon[0,0],lon[1,1],lat[-1,-1],lat[0,0]))
        elif types == "contourf":
            sct = projs.contourf(lon, lat,radar, cmap = radarcm["cmap"], norm=radarcm["norm"], levels = radarcm["levels"],latlon=True)
        plt.colorbar(sct)
        plt.title(obstime,fontsize=18)
    if maincat == "A0001" or fn == "O-A0001-001":
        outfn = O_A0001_001.apiget(apipath)
        fig, projs = O_A0001_001_vis.O_A0001_VIS(outfn, metvars)
    return fig, projs

def json_api(fn,apipath):
    if fn == "O-A0001-001":
       outfn = O_A0001_001.apiget(apipath)
    elif fn == "O-A0002-001":
       outfn = O_A0002_001.apiget(apipath)
    elif fn == "O-A0003-001":
       outfn = O_A0003_001.apiget(apipath)
    elif fn == "O-B0045-001":
       outfn = O_B0045_001.apiget(apipath)
    return outfn

def read_grib(fn):
    outfn = pb.open(fn)
    return outfn



import matplotlib.colors as mcolors

def lightrain(tranp=0.5):
    rain_light_temp = [[1.        , 1.        , 1.        , 0.5       ],
       [0.77647059, 0.77254902, 0.76470588, 0.5       ],
       [0.60784314, 1.        , 1.        , 0.5       ],
       [0.        , 0.81176471, 1.        , 0.5       ],
       [0.00392157, 0.59607843, 1.        , 0.5       ],
       [0.00392157, 0.39607843, 1.        , 0.5       ],
       [0.18823529, 0.6       , 0.00392157, 0.5       ],
       [0.19607843, 1.        , 0.        , 0.5       ],
       [0.97254902, 1.        , 0.        , 0.5       ],
       [1.        , 0.79607843, 0.        , 0.5       ],
       [1.        , 0.60392157, 0.        , 0.5       ],
       [0.98039216, 0.01176471, 0.        , 0.5       ],
       [0.8       , 0.        , 0.01176471, 0.5       ],
       [0.62745098, 0.        , 0.        , 0.5       ],
       [0.59607843, 0.        , 0.60392157, 0.5       ],
       [0.76470588, 0.01568627, 0.8       , 0.5       ],
       [0.97254902, 0.01960784, 0.95294118, 0.5       ],
       [0.99607843, 0.79607843, 1.        , 0.5       ]]
    rain_light = rain_light_temp.copy()
    for rgbspos in rain_light:
        rgbspos[3] = tranp
    return rain_light

def rain(colorlevel = [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400], style = 'nwprfs'):
    if style == "nwprfs":
        cwb_data=['None','#C6C5C3','#9BFFFF','#00CFFF','#0198FF','#0165FF','#309901','#32FF00','#F8FF00','#FFCB00',\
               '#FF9A00','#FA0300','#CC0003', '#A00000','#98009A','#C304CC','#F805F3','#FECBFF']
        cmaps = mcolors.ListedColormap(cwb_data,'precipitation')
        numticks = len(cwb_data) +1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]")
            colorlevel = [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    elif style == "nwprfs_light1":
        cwb_data = lightrain(tranp=0.5)
        cmaps = mcolors.ListedColormap(cwb_data,'precipitation')
        numticks = len(cwb_data) +1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]")
            colorlevel = [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    elif style == "nwprfs_light2":
        cwb_data = lightrain(tranp=0.3)
        cmaps = mcolors.ListedColormap(cwb_data,'precipitation')
        numticks = len(cwb_data) +1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]")
            colorlevel = [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    elif style.lower() == 'npd':
        colorlevelnpd = [0,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]
        cwb_data=['None','#9BFFFF','#00CFFF','#0198FF','#0165FF','#309901','#32FF00','#F8FF00','#FFCB00',\
               '#FF9A00','#FA0300','#CC0003', '#A00000','#98009A','#C304CC','#F805F3','#FECBFF']
        cmaps = mcolors.ListedColormap(cwb_data,'precipitation')
        numticks = len(cwb_data) +1
        if colorlevel != colorlevelnpd and colorlevel == [0,0.1,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]:
            colorlevel = colorlevelnpd
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        elif len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [0,1,2,6,10,15,20,30,40,50,70,90,110,130,150,200,300,400]")
            colorlevel = colorlevelnpd
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    dictobj ={"levels":colorlevel,"norm":norms,"cmap":cmaps}
    return dictobj

def radar(colorlevel = [-1000, -900, 0 ,5,10,15,20,25,30,35,40,45,50,55,60,65,70], style='mosaic'):
    if style.lower() == 'cwbweb':
        cwb_data = ["#84C1FF","#2894FF","#0066CC","#28FF28","#00BB00","#009100","#F9F900","#FF8000","#FF5151","#EA0000","#AE0000","#FF44FF","#D200D2","#750075"]
        cmaps = mcolors.ListedColormap(cwb_data,'radar')
        coloraschen = [5,10,15,20,25,30,35,40,45,50,55,60,65,70,75]
        numticks = len(cwb_data) +1 
        if colorlevel != coloraschen and colorlevel == [-1000, -900, 0 ,5,10,15,20,25,30,35,40,45,50,55,60,65,70]:
            colorlevel = coloraschen
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        elif len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [5,10,15,20,25,30,35,40,45,50,55,60,65,70,75]")
            colorlevel = [5,10,15,20,25,30,35,40,45,50,55,60,65,70,75]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    elif style.lower() == "mosaic":
        cwb_data = ["#F2F2F2","white","#07FDFD","#0695FD", "#0203F9","#00FF00","#00C800","#019500","#FEFD02","#FEC801","#FD7A00","#FB0100","#C70100","#950100","#FA03FA","#9800F6"]
        cmaps = mcolors.ListedColormap(cwb_data,'radar')
        numticks = len(cwb_data) + 1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [-1000, -900, 0 ,5,10,15,20,25,30,35,40,45,50,55,60,65,70]")
            colorlevel = [-1000, -900, 0 ,5,10,15,20,25,30,35,40,45,50,55,60,65,70]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        #print("finish yet")
    dictobj = {"levels":colorlevel,"norm":norms,"cmap":cmaps}
    return dictobj


def surfT(colorlevel = list(range(-1,40)),style='cwbweb'):
    if style == 'cwbweb':
        cwb_data=['#117388','#207E92','#2E899C','#3D93A6','#4C9EB0','#5BA9BA','#69B4C4','#78BFCE','#87CAD8','#96D4E2','#A4DFEC','#B3EAF6','#0C924B','#1D9A51','#2FA257','#40A95E','#51B164','#62B96A','#74C170','#85C876','#96D07C','#A7D883','#B9E089','#CAE78F','#DBEF95','#F4F4C3','#F7E78A','#F4D576','#F1C362','#EEB14E','#EA9E3A','#E78C26','#E07B03','#ED5138','#ED1759','#AD053A','#780101','#9C68AD','#845194','#8520A0']
        cmaps = mcolors.ListedColormap(cwb_data,'surfT')
        numticks = len(cwb_data) +1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [5,10,15,20,25,30,35,40,45,50,55,60,65,70,75]")
            colorlevel = list(range(-2,41))
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    dictobj ={"norm":norms,"cmap":cmaps,"levels":colorlevel}
    return dictobj


def rh(colorlevel=[60,65,70,75,80,85,90,95,100],style="npd"):
    if style.lower() == "npd":
        cwb_data = ["None", "#D3E6EB", "#A7CFD8","#82F550","#4ADC0C","#93F4FF","#2DEAFF","#02D4E3"]
        cmaps = mcolors.ListedColormap(cwb_data,'rh')
        numticks = len(cwb_data) +1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [60,65,70,75,80,85,90,95,100]")
            colorlevel = [60,65,70,75,80,85,90,95,100]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    if style.lower() == "rhcc":
        cwb_data = ["None", "#DAD892", "#A0E3B7", "#21F0B6", "#7ED4D8" , "#4BD6FD", "#2C928B", "#25738B" ]
        cmaps = mcolors.ListedColormap(cwb_data,'rh')
        numticks = len(cwb_data) +1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set [60,65,70,75,80,85,90,95,100]")
            colorlevel = [60,65,70,75,80,85,90,95,100]
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel, cmaps.N)
    dictobj = {"norm":norms,"cmap":cmaps,"levels":colorlevel}
    return dictobj

def freq(colorlevel = [1, 2, 5, 8, 15, 20, 25, 30, 35, 40, 100, 300, 500], plttype="hist2d", style="ccliu"):
    #Some color is took from imola colormap in the proplot package.
    defclevel = "[1, 2, 5, 8, 15, 20, 25, 30, 35, 40, 100, 300, 500]"
    if style.lower() == "ccliu":
        cwb_data = ['#2446a9', '#2d59a0', '#396b94', '#497b85', '#60927b', '#7bae74', '#99cc6d', '#c4ea67', '#ffff66', '#FFCB00', '#FF9A00', 'orangered']
        cmaps = mcolors.ListedColormap(cwb_data,'freq')
        numticks = len(cwb_data) + 1
        if len(colorlevel) != numticks:
            print("the length of colorlevel (len(colorlevel)) need {:d}.".format(numticks))
            print("Now use default set {}".format(defclevel))
            colorlevel = [1, 2, 5, 8, 15, 20, 25, 30, 35, 40, 100, 300, 500]
            norms = mcolors.BoundaryNorm(colorlevel,cmaps.N)
        else:
            norms = mcolors.BoundaryNorm(colorlevel,cmaps.N)
    if plttype == "hist2d":
        dictobj = {"norm":norms,"cmap":cmaps}
    else:
        dictobj = {"norm":norms,"cmap":cmaps,"levels":colorlevel}
    return dictobj



import os
from setuptools import setup, find_packages
# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), fname), "r") as fin:
        return fin.read()
setup(
    name = "cwbplot",
    version = "0.0.8",
    author = "cwbrfsteam",
    author_email = "littlepon4@hotmail.com",
    description = "Central weather bureau rfs team and open data useful tool for daily wrok.",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    license = "MIT",
    url = "https://github.com/Liu-CWB/cwbplot",
    project_urls={
        # 'Documentation': 'https://packaging.python.org/tutorials/distributing-packages/',
        # 'Funding': 'https://donate.pypi.org',
        'Source': "https://github.com/Liu-CWB/cwbplot",
        # 'Tracker': 'https://github.com/pypa/sampleproject/issues',
    },
    #packages=find_packages(exclude=["tests*"]),
    packages=['cwbplot'],
    package_dir={'cwbplot': 'src/cwbplot'},
    package_data={'cwbplot': ['fonts/*.ttf','sharedata/*'],},
    python_requires=">=3.6",
    install_requires=[
        "pandas>=1.0.3",
        "numpy>=1.17.3",
        "matplotlib>=3.3.3,<3.5.0",
        "pygrib>=2.0.4",
        "requests>=2.25.0",
        "netcdf4>=1.5.6,<=1.6.0",
        "basemap>=1.3.0",
        "basemap-data>=1.3.0",
        "basemap-data-hires>=1.3.0",
        "xmltodict>=0.12.0",
    ],
    extras_require={
                 'geotiff':['gdal>=2.4.1']
                   },
    #setup_requires=[
    #     "pandas>=1.0.3",
    # ],
    # see http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 1 - Planning",
        #"Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3 :: Only",
    ],)



from matplotlib.lines import Line2D

def filled(maxws, marker='o', linestyle ='-', sizes=5):
    if maxws <= 62/3.6 and maxws >= 41/3.6:
        #markdict = dict(marker=marker, s=sizes, c='dodgerblue', edgecolors='darkblue',alpha=0.9)
        linedict = dict(linestyle=linestyle, marker=marker, markersize=sizes, markeredgecolor='darkblue',markerfacecolor='dodgerblue',color='darkblue', alpha=0.90)
    elif maxws > 41/3.6 and maxws <= 32.6:
        #markdict = dict(marker=marker, s=sizes, c='coral', edgecolors='#E35200', alpha=0.9)
        linedict = dict(linestyle=linestyle, marker=marker,  markersize=sizes, markeredgecolor='#E35200',markerfacecolor='coral',color='#E35200', alpha=0.90)
    elif maxws > 32.6 and maxws <= 50.9:
        #markdict = dict(marker=marker, s=sizes, c='indianred', edgecolors='maroon', alpha=0.9)
        linedict = dict(linestyle=linestyle, marker=marker,  markersize=sizes, markeredgecolor='maroon',markerfacecolor='indianred',color='maroon', alpha=0.90)
    elif maxws > 50.9:
        #markdict = dict(marker=marker, s=sizes, c='orchid', edgecolors='purple', alpha=0.9)
        linedict = dict(linestyle=linestyle, marker=marker, markersize=sizes , markeredgecolor='purple',markerfacecolor='orchid',color='purple', alpha=0.90)
    else:
        #markdict = dict(marker=marker, s=sizes, c='white',edgecolors='dimgray', alpha=0.9)
        linedict = dict(linestyle=linestyle, marker=marker, markersize=sizes , markeredgecolor='dimgray',markerfacecolor='white',color='dimgray', alpha=0.90)
    nontroplin2d = Line2D([0],[0], linestyle=linestyle, color = "dimgray", marker=marker,markersize=5, markeredgecolor='dimgray' , markerfacecolor='white', alpha=0.9)
    td2d = Line2D([0],[0], linestyle = linestyle, color = "darkblue", marker=marker, markersize=5, markeredgecolor='darkblue' , markerfacecolor='dodgerblue', alpha=0.9)
    mild2d = Line2D([0],[0], linestyle = linestyle, color = "#E35200", marker=marker, markersize=5, markeredgecolor='#E35200' , markerfacecolor='coral', alpha=0.9)
    mode2d = Line2D([0],[0], linestyle = linestyle, color = "maroon", marker=marker, markersize=5, markeredgecolor='maroon' , markerfacecolor='indianred', alpha=0.9)
    ser2d = Line2D([0],[0], linestyle = linestyle, color = "purple", marker=marker, markersize=5, markeredgecolor='purple' , markerfacecolor='orchid', alpha=0.9)
    handle_elements = [nontroplin2d, td2d, mild2d, mode2d, ser2d]
    lable_elements = ["Non-Tropical","TD","Mild","Moderate","Severe"]
    legend_elements = [handle_elements, lable_elements]
    return linedict, legend_elements

